//+------------------------------------------------------------------+
//|                                         Tick Volume Indicator v2 |
//|                                         Copyright  William Blau |
//|                           Originally coded  2006 by Profitrader |
//|                                       Recoded  2011 by MaryJane |
//+------------------------------------------------------------------+
#property copyright "Profitrader, 2006 && MaryJane, 2011"
#property strict
//+------------------------------------------------------------------+
//---- properties
#property indicator_separate_window
#property indicator_buffers 5
#property indicator_color1 DodgerBlue     // for line graph, set color1-color4
#property indicator_color2 Red            // to CLR_NONE = set None in user
#property indicator_color3 DodgerBlue     // settings.
#property indicator_color4 Red            //
#property indicator_color5 DarkGray          // set CLR_NONE to hide the line
#property indicator_width1 3
#property indicator_width2 3
#property indicator_width3 3
#property indicator_width4 3
#property indicator_width5 3
//+------------------------------------------------------------------+
//---- input parameters
extern string TimeFrame          = "current time frame";
extern int     r                 = 12;
extern int     s                 = 12;
extern int     u                 = 5;
extern string  _n2               = "_AlertCandle = 0: current";
extern string  _n3               = "_AlertCandle = 1: closed";
extern int     AlertCandle       = 1;
extern bool    PopupAlerts       = false; 
extern bool    AlertSound        = false;
extern string  _n4               = "_Leave empty for no email";
extern string  AlertEmailSubject = "";
//+------------------------------------------------------------------+
//---- globalscape
datetime       LastAlertTime     = -999999;
string         AlertUp           = "TVI change: UP";
string         AlertDn           = "TVI change: DOWN";
int            Precision         = 5;
datetime       OldTime;
string         ShortName;
string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;
//+------------------------------------------------------------------+
//---- buffers
double UpPos[], DnPos[], UpNeg[], DnNeg[];
double TVI[], Trend[];
//+------------------------------------------------------------------+
//---- extra buffers
//+------------------------------------------------------------------+
//| indicator initialization                                         |
//+------------------------------------------------------------------+
int init()
{ 
   IndicatorDigits(Precision);
   IndicatorBuffers(6);
   SetIndexBuffer(0, UpPos); SetIndexStyle(0, DRAW_HISTOGRAM); 
   SetIndexBuffer(1, DnPos); SetIndexStyle(1, DRAW_HISTOGRAM);
   SetIndexBuffer(2, UpNeg); SetIndexStyle(2, DRAW_HISTOGRAM);
   SetIndexBuffer(3, DnNeg); SetIndexStyle(3, DRAW_HISTOGRAM);
   SetIndexBuffer(4, TVI);   
   SetIndexBuffer(5, Trend);
   
   ShortName = "TVI v2 (" + (string)r + "," + (string)s + "," + (string)u + ") Trend is ";
   
         indicatorFileName = WindowExpertName();
         timeFrame = stringToTimeFrame(TimeFrame);
         returnBars        = TimeFrame=="returnBars";     if (returnBars)     { return(0); }
         calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) { return(0); }
   
   return(0);
}

//+------------------------------------------------------------------+
//| indicator code                                                   |
//+------------------------------------------------------------------+
int start()
   {
   int i;
   string t;
   int counted_bars = IndicatorCounted();
      if (counted_bars < 0) return (-1);
      if (counted_bars > 0) counted_bars--;
         int limit=MathMin(Bars-counted_bars,Bars-1)-1;
         if (returnBars) { UpPos[0] = limit+1; return(0); }

   if (calculateValue || timeFrame==Period())
   {
         for(i = limit; i >= 0; i--)
         {
            double UpTicks    = (Volume[i] + (Close[i] - Open[i]) / Point) / 2;
            double DnTicks    =  Volume[i] - UpTicks;
            double EMA_U      = iEma(UpTicks,r,i,0);
            double EMA_D      = iEma(DnTicks,r,i,1);
            double DEMA_U     = iEma(EMA_U  ,s,i,2);
            double DEMA_D     = iEma(EMA_D  ,s,i,3);
            double TVI_Raw    = 100.0 * (DEMA_U - DEMA_D) / (DEMA_U + DEMA_D);
                   TVI[i]     = iEma(TVI_Raw,u,i,4);
                   Trend[i]   = Trend[i+1];
                     if (TVI[i] > TVI[i+1]) Trend[i] =  1;
                     if (TVI[i] < TVI[i+1]) Trend[i] = -1;
                     if (Trend[i] > 0)
                     {
                        if (TVI[i] >= 0) 
                              {UpPos[i] = TVI[i]; DnPos[i] = EMPTY_VALUE;}
                        else  {UpNeg[i] = TVI[i]; DnNeg[i] = EMPTY_VALUE;}
                     }
                     if (Trend[i] < 0)
                     {
                        if (TVI[i] >= 0) 
                              {DnPos[i] = TVI[i]; UpPos[i] = EMPTY_VALUE;}
                        else  {DnNeg[i] = TVI[i]; UpNeg[i] = EMPTY_VALUE;}
                     }
         }
         //---- trend display update
         if (Trend[0] == 1) t = "UP"; else t = "DOWN"; 
         IndicatorShortName(ShortName + t); 
         
         //---- do alerts
         ProcessAlerts();         
         return(0);
      }
      
   //
   //
   //
   //
   //
   
   limit = (int)MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         UpPos[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",r,s,u,"","",AlertCandle,PopupAlerts,"",AlertEmailSubject,0,y);
         DnPos[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",r,s,u,"","",AlertCandle,PopupAlerts,"",AlertEmailSubject,1,y);
         UpNeg[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",r,s,u,"","",AlertCandle,PopupAlerts,"",AlertEmailSubject,2,y);
         DnNeg[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",r,s,u,"","",AlertCandle,PopupAlerts,"",AlertEmailSubject,3,y);
         TVI[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",r,s,u,"","",AlertCandle,PopupAlerts,"",AlertEmailSubject,4,y);
   }
   return(0);
}

double workEma[][6];
double iEma(double price, double period, int tr, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); tr = Bars-tr-1;
   if (tr==0) { workEma[tr][instanceNo] = price; return(price); }

   //
   //
   //
   //
   //
      
   double alpha = 2.0 / (1.0+period);
          workEma[tr][instanceNo] = workEma[tr-1][instanceNo]+alpha*(price-workEma[tr-1][instanceNo]);
   return(workEma[tr][instanceNo]);
}

//+------------------------------------------------------------------+
//| alert routine (thank you hanover :-)                             |
//| http://www.forexfactory.com/showthread.php?t=299520              |
//+------------------------------------------------------------------+
void ProcessAlerts()
   {               
   string AlertText;
   if (AlertCandle >= 0  &&  Time[0] > LastAlertTime)
      {
      //---- alert UP
      if (Trend[AlertCandle] == 1 &&  Trend[AlertCandle + 1] != 1)
         {
         AlertText = Symbol() + "," + TFToStr(Period()) + ": " + AlertUp;
         if (PopupAlerts) Alert(AlertText);
         if (AlertEmailSubject > "") SendMail(AlertEmailSubject, AlertText);
         LastAlertTime = Time[0];
         if (AlertSound) PlaySound("alert2.wav"); 
         }
      //---- alert DOWN
      if (Trend[AlertCandle] == - 1  &&  Trend[AlertCandle + 1] != -1)
         {
         AlertText = Symbol() + "," + TFToStr(Period()) + ": " + AlertDn;
         if (PopupAlerts) Alert(AlertText);
         if (AlertEmailSubject > "") SendMail(AlertEmailSubject, AlertText);
         LastAlertTime = Time[0];
         if (AlertSound) PlaySound("alert2.wav"); 
         }
      }
   }
//+------------------------------------------------------------------+
string TFToStr(int tf)
   {
   if (tf == 0)        tf = Period();
   if (tf >= 43200)    return("MN"); 
   if (tf >= 10080)    return("W1"); 
   if (tf >=  1440)    return("D1"); 
   if (tf >=   240)    return("H4"); 
   if (tf >=    60)    return("H1"); 
   if (tf >=    30)    return("M30");
   if (tf >=    15)    return("M15");
   if (tf >=     5)    return("M5"); 
   if (tf >=     1)    return("M1"); 
   return("");
   }        
//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+(string)iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   ts = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(ts, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     ts = StringSetChar(ts, length, (ushort)(tchar - 32));
         else if(tchar > -33 && tchar < 0)
                     ts = StringSetChar(ts, length, (ushort)(tchar + 224));
   }
   return(ts);
}